--[[
************************************************************************

tracker.lua

File date: 2019-12-04T19:15:01Z
File revision: 
Project revision: 291
Project version: r291


Author: Angelike

************************************************************************
]]--

local beql = LibStub("AceAddon-3.0"):GetAddon("beql")
local L = LibStub("AceLocale-3.0"):GetLocale("beql")

-- local functions

local displayQuest, displayQuestObjective, displayTrackerZone, getQuestTrackerLine, displayTrackerHeader, displayTrackerFrame, displayQuestTimer;

function beql:initQuestTracker()
	beql.frames.questWatchAnchor:SetScript('OnUpdate', beql.QuestWatchAnchor_OnUpdate)
	beql.frames.questWatchAnchor:RegisterEvent('QUEST_LOG_UPDATE');
	beql.frames.questWatchAnchor:RegisterEvent('QUEST_WATCH_UPDATE');

    beql.frames.questWatchFrame:SetScript('OnDragStart', beql.watchFrame_OnDragStart);
    beql.frames.questWatchFrame:SetScript('OnDragStop', beql.watchFrame_OnDragStop);
	beql.frames.questWatchFrame:SetScript('OnUpdate', beql.QuestTracker_OnUpdate);
    beql.frames.questWatchFrame.updateTooltip = beql.QuestTracker_OnUpdate;

    beql.frames.questWatchTitle:SetScript('OnDragStart', beql.watchFrame_OnDragStart);
    beql.frames.questWatchTitle:SetScript('OnDragStop', beql.watchFrame_OnDragStop);

    beql.frames.questWatchToggleButton:SetScript('OnDragStart', beql.watchFrame_OnDragStart);
    beql.frames.questWatchToggleButton:SetScript('OnDragStop', beql.watchFrame_OnDragStop);
	beql.frames.questWatchToggleButton:SetScript('OnClick', beql.questWatchFrame_OnToggle);

    beql.db.char.QuestWatchIDs = newArray(unpack(beql.db.char.QuestWatchIDs));

    beql:On('QUEST_ADDED', beql.autoWatchNewQuest);
    beql:On('QUEST_REMOVED', beql.questWatchRemoveQuest);
    beql:On('QUEST_LOG_CHANGED', beql.QuestTracker_ScheduleUpdate);
end

function beql:questWatchFrame_OnToggle(...)
    if(select('#',...) > 0) then
        -- clicked
        beql.db.char.QuestWatchMinimized = not beql.db.char.QuestWatchMinimized;
    end

    if (beql.db.char.QuestWatchMinimized) then
        beql.frames.questWatchToggleButton:SetNormalTexture("Interface\\AddOns\\beql\\Images\\restore_up")
        beql.frames.questWatchToggleButton:SetPushedTexture("Interface\\AddOns\\beql\\Images\\restore_down")
        beql.frames.questWatchFrame:SetAlpha(0);
    else
        beql.frames.questWatchToggleButton:SetNormalTexture("Interface\\AddOns\\beql\\Images\\minimize_up")
        beql.frames.questWatchToggleButton:SetPushedTexture("Interface\\AddOns\\beql\\Images\\minimize_down")
        beql.frames.questWatchFrame:SetAlpha(beql.db.profile.QuestTracker.Alpha);
    end

	beql:QuestTracker_ScheduleUpdate();
	beql.frames.questWatchAnchor.updateTime = 0.002;
end

function beql:autoWatchQuestProgress(index)
    if (IsQuestWatched(index) or beql.db.profile.QuestTracker.AddUntracked ~= true) then
        return;
    end

    beql:toggleQuestWatch(index);
end

function beql:autoWatchNewQuest(questID)
    if (beql.db.char.QuestWatchIDs.includes(questID) or beql.db.profile.QuestTracker.AddNew ~= true) then
        return;
    end 

    beql.db.char.QuestWatchIDs.push(questID);
    beql:QuestTracker_SortWatchedQuests();
end

function beql:questWatchRemoveQuest(questID)
    local watchIDs = beql.db.char.QuestWatchIDs

    if (watchIDs.includes(questID)) then
        watchIDs.splice(watchIDs.indexOf(questID), 1);
    end 
end

-- Toggles Questtracker status between active and inactive
function beql:QuestTracker_ToggleStatus(inactive)
	beql.frames.questWatchFrame:Hide();

	if (not inactive) then
		beql:QuestTracker_ScheduleUpdate()
	end

	QuestLog_Update();
end

function beql:toggleQuestWatch(index)
    local watched = IsQuestWatched(index); -- <-- api hooked !
    local zone, _, _, isHeader, _, _, _, questID = GetQuestLogTitle(index);

    if (isHeader) then
        local zoneQuests = beql:GetZoneQuests(zone);
        local zoneWatched = true;
        for _, childID in pairs(zoneQuests) do
            local childIndex = select(9, beql:GetQuestLogTitle(childID));

            zoneWatched = zoneWatched and IsQuestWatched(childIndex);
        end

        for _, childID in pairs(zoneQuests) do
            local childIndex = select(9, beql:GetQuestLogTitle(childID));

            if(IsQuestWatched(childIndex) == zoneWatched) then
                beql:toggleQuestWatch(childIndex);
            end
        end
    elseif (watched) then
        for k, id in ipairs(beql.db.char.QuestWatchIDs) do
            if (id == questID) then
                tremove(beql.db.char.QuestWatchIDs, k);
                break;
            end
        end
        index = nil
    else
        beql.db.char.QuestWatchIDs.push(questID);
        beql:QuestTracker_SortWatchedQuests();
    end

    beql:QuestTracker_DisplayQuests();
    QuestLog_Update();
end

function beql:IsQuestWatched(index)
    local _, _, _, _, _, _, _, questID = GetQuestLogTitle(index);

    return beql.db.char.QuestWatchIDs.includes(questID);
end

function beql:QuestTracker_SortWatchedQuests()
    if (beql.db.profile.QuestTracker.SortItems ~= true) then
        return;
    end

    table.sort(beql.db.char.QuestWatchIDs, function(a, b)
        local aText, aLevel, _, _, _, _, _, _, _, aHeader = beql:GetQuestLogTitle(a);
        local bText, bLevel, _, _, _, _, _, _, _, bHeader = beql:GetQuestLogTitle(b);

        if (beql.db.profile.QuestTracker.ShowZones and aHeader ~= bHeader) then
            return aHeader < bHeader;
        end

        if (beql.db.profile.QuestLog.showlevel and aLevel ~= bLevel) then
            return aLevel < bLevel;
        end

        return aText < bText;
    end)
end

-- Update Base Stats of the Frame like Color etc...
local FontHeight, FontSpacing
function beql:QuestTracker_UpdateFrame()
	
	FontHeight, FontSpacing, QuestSpacing, ZoneSpacing = nil, nil, nil, nil
	local QuestTrackerOptions = beql.db.profile.QuestTracker

	beql:debugprint("beql:QuestTracker_UpdateFrame");

	-- movability
	if (QuestTrackerOptions.locked) then
        beql.frames.questWatchTitle:EnableMouse(false);
        beql.frames.questWatchTitle:RegisterForDrag(nil);
		beql.frames.questWatchFrame:RegisterForDrag(nil)
		beql.frames.questWatchAnchor:SetMovable(nil);
	else
        beql.frames.questWatchTitle:EnableMouse(true);
        beql.frames.questWatchTitle:RegisterForDrag("LeftButton");
		beql.frames.questWatchFrame:RegisterForDrag("LeftButton");
		beql.frames.questWatchAnchor:SetMovable(true);
	end

	-- Title
	if (QuestTrackerOptions.ShowHeader) then
		beql.frames.questWatchTitle:Show();
	else
		beql.frames.questWatchTitle:Hide();
	end

    local borderColor, bgColor = CreateColor(0, 0, 0, 0), CreateColor(0, 0, 0, 0);

	if (QuestTrackerOptions.UseBgColors) then
		if (QuestTrackerOptions.CustomBgColor) then
            borderColor = QuestTrackerOptions.Colors.BgBorder;
            bgColor = QuestTrackerOptions.Colors.Bg;
		else
            borderColor = TOOLTIP_DEFAULT_BACKGROUND_COLOR;
            bgColor = TOOLTIP_DEFAULT_BACKGROUND_COLOR;
		end
	end

    beql.frames.questWatchTitle:SetBackdropBorderColor(borderColor.r, borderColor.g, borderColor.b, borderColor.a)
    beql.frames.questWatchTitle:SetBackdropColor(bgColor.r, bgColor.g, bgColor.b, bgColor.a)
    beql.frames.questWatchFrame:SetBackdropBorderColor(borderColor.r, borderColor.g, borderColor.b, borderColor.a)
    beql.frames.questWatchFrame:SetBackdropColor(bgColor.r, bgColor.g, bgColor.b, bgColor.a)

	local t1, _, t2 = beql.frames.questWatchTitle.Text:GetFont();
	beql.frames.questWatchTitle.Text:SetFont(t1, QuestTrackerOptions.FontHeight + 2, t2)
	beql.frames.questWatchTitle.Text:SetHeight(QuestTrackerOptions.FontHeight + 2)

	-- Alpha
	beql.frames.questWatchFrame:SetAlpha(beql.db.profile.QuestTracker.Alpha)
	
	if (beql.frames.questWatchFrame:IsVisible()) then
		beql:QuestTracker_DisplayQuests();
	end

	return 1;
end

function beql:QuestTracker_ScheduleUpdate()
    beql.frames.questWatchAnchor.updateTime = 0.02;
    beql.frames.questWatchAnchor:SetScript('OnUpdate', beql.QuestWatchAnchor_OnUpdate);
end

function beql:QuestWatchAnchor_OnUpdate(elapsed)
	if (self.UIScale ~= UIParent:GetScale()) then
		beql:RestoreFramePosition(self);
	end

    self.updateTime = (self.updateTime or 0) - elapsed

    if (self.updateTime > 0) then
        return;
    end

    self:SetScript('OnUpdate', nil);
    beql:QuestTracker_UpdateFrame();
    beql:QuestTracker_DisplayQuests();
end

--
-- Hooks
--

local WATCHFRAMELINES_FONTSPACING = 0

function beql:QuestTracker_DisplayQuests()
    beql.db.char.QuestWatchIDs = beql.db.char.QuestWatchIDs.filter(function(questID)
        return beql:IsQuestValid(questID);
    end)

    beql.frames.watchLineCache:Reset();

	if (not beql:IsEnabled() or beql.db.profile.disabledtracker or #beql.db.char.QuestWatchIDs == 0) then
		beql.frames.questWatchFrame:Hide()

		return
	end
	
	-- beql locales
	local questIndex, curZone;
	local qTitle, qLvl, qTag, qRec, qStat, qObj, qZone, qID, qdaily

    displayTrackerHeader()

	for _, questID in pairs(beql.db.char.QuestWatchIDs) do
        local _, _, _, _, _, isComplete, _, _, questIndex, qZone = beql:GetQuestLogTitle(questID);

        --
        -- Zone !
        --
        curZone = displayTrackerZone(curZone, qZone, isComplete);

        --
        -- Quest !
        --
        displayQuest(questIndex);

        --
        -- Timer
        --
        displayQuestTimer(questID);
	end

	-- Handle Frame Visibility
    beql.frames.questWatchFrame:Show()

    displayTrackerFrame();
end

function displayTrackerHeader()
    local _, index, isComplete, numQuests, questEntries;
    local completedQuests = 0;

    for _, questID in ipairs(beql.db.char.QuestWatchIDs) do
        _, _, _, _, _, isComplete = beql:GetQuestLogTitle(questID);

        if (isComplete) then
            completedQuests = completedQuests + 1
        end
    end

	numQuests = #beql.db.char.QuestWatchIDs;

	_, questEntries = GetNumQuestLogEntries()
	
    if (beql.db.profile.QuestTracker.RemoveFinished) then
		beql.frames.questWatchTitle.Text:SetText(L["Quest Tracker"] .. " ("..numQuests - completedQuests.."/"..questEntries..")")
	else
		beql.frames.questWatchTitle.Text:SetText(L["Quest Tracker"] .. " ("..numQuests.."/"..questEntries..")")
	end

    -- set Tracker Width
    beql.frames.questWatchFrame:SetWidth(beql.frames.questWatchTitle.Text:GetStringWidth() + beql.db.profile.QuestTracker.FontHeight + 24 + 2);
end

function displayTrackerZone(lastZone, newZone, complete)
    local color, line;

    if (not beql.db.profile.QuestTracker.ShowZones or beql.db.profile.QuestTracker.RemoveFinished == complete or lastZone == newZone or beql.db.char.QuestWatchMinimized) then
        return lastZone;
    end

    if (beql.db.profile.QuestTracker.CustomZoneColor) then
        color = beql.db.profile.QuestTracker.Colors.Zone
    else
        color = {r = 0.8, g = 0.8, b = 1}
    end

    line = getQuestTrackerLine(lines);
    line.type = 'header';
    line.questID = newZone;
    line.text:SetText(newZone);
    line.text:SetTextColor(color.r, color.g, color.b, color.a)

    return newZone;
end

function displayQuest(questIndex)
    local title, qLvl, tag, isHeader, isCollapsed, isComplete, frequency, questID = GetQuestLogTitle(questIndex);

    local Color, line;

    if (beql.db.profile.QuestTracker.RemoveFinished and isComplete == 1 or beql.db.char.QuestWatchMinimized) then
        -- complete and hidden, exit
        return;
    end

    -- Colores
    if (isComplete == 1) then
        --Completed
        if (beql.db.profile.QuestTracker.CustomHeaderColor) then
            if (beql.db.profile.QuestTracker.HeaderQuestLevelColor) then
                Color = GetQuestDifficultyColor(qLvl)
            else
                Color = beql.db.profile.QuestTracker.Colors.HeaderComplete
            end
        else
            Color = NORMAL_FONT_COLOR
        end
    elseif (isComplete == -1) then
        -- Failed
        if (beql.db.profile.QuestTracker.CustomHeaderColor) then
            Color = beql.db.profile.QuestTracker.Colors.HeaderFailed
        else
            Color = NORMAL_FONT_COLOR
        end
    else
        -- Normal
        if (beql.db.profile.QuestTracker.CustomHeaderColor) then
            if (beql.db.profile.QuestTracker.HeaderQuestLevelColor) then
                Color = GetQuestDifficultyColor(qLvl)
            else
                Color = beql.db.profile.QuestTracker.Colors.HeaderNormal
            end
        else
            Color = { r = 0.75, g = 0.61, b = 0 }
        end
    end

    line = getQuestTrackerLine();
    if (beql.db.profile.QuestTracker.ShowZones) then
        line.dash:SetText("  ")
    end

    line.type = 'quest';
    line.text:SetText(title);
    line.questID = questID;
    line.text:SetTextColor(Color.r, Color.g, Color.b)
    line:Show();
    
    --
    -- Objectives
    --

    if (beql.db.profile.QuestTracker.MinimizeFinished or beql.db.profile.QuestTracker.HideCompletedOnly) and isComplete == 1 then
        -- hide questobjectives on complete quest
        return;
    end

    for i = 1, GetNumQuestLeaderBoards(questIndex) do
        displayQuestObjective(questIndex, i);
    end
end

function displayQuestObjective(questIndex, objectiveIndex)
    local text, otype, complete, got, need = GetQuestLogLeaderBoard(objectiveIndex, questIndex);
    local _, _, _, _, _, _, _, questID = GetQuestLogTitle(questIndex);

    local numbers, tempColor, tempColor2, Color, objText;

    objText = '';

    if (complete and beql.db.profile.QuestTracker.MinimizeFinished) then
        -- hide complete
        return;
    end

    local numbers = {text:match('(%d+)/(%d+)')};

    if (#numbers > 0) then
        got, need = unpack(numbers);
    elseif (complete) then
        got, need = 1, 1;
    else
        got, need = 0, 1;
    end

    -- colores
    if (beql.db.profile.QuestTracker.CustomObjetiveColor) then
        tempColor = beql.db.profile.QuestTracker.Colors.ObjectiveNormal;
        tempColor2 = beql.db.profile.QuestTracker.Colors.ObjectiveComplete;
    else
        tempColor = CreateColor(0.8, 0.8, 0.8);
        tempColor2 = HIGHLIGHT_FONT_COLOR;
    end

    if (complete) then
        Color = tempColor2
    elseif (beql.db.profile.QuestTracker.FadeColor) then
        Color = beql:FadeColors(tempColor, tempColor2, got, need)
    else
        Color = tempColor
    end

    -- markers
    if (beql.db.profile.QuestTracker.ShowObjectiveMarkers) then
        if (beql.db.profile.QuestTracker.UseTrackerListing) then
            objText = " "..BEQL_TrackerLists[beql.db.profile.QuestTracker.TrackerList][objectiveIndex]..") "
        else
            objText = " "..BEQL_TrackerSymbols[beql.db.profile.QuestTracker.TrackerSymbol].." "
        end
    else
        if (IsAddOnLoaded("Mobmap")) then
            objText = "- "
        else
            objText = "  "
        end
    end

    objText = objText .. text

    -- create line
    line = getQuestTrackerLine();
    line.type = 'objective';
    line.questID = questID;

    if (beql.db.profile.QuestTracker.ShowZones) then
        line.dash:SetText("  ")
    end

    line.text:SetText(objText);
    line.text:SetTextColor(Color.r, Color.g, Color.b)
    line:Show();
end

function displayQuestTimer(questID)
    if (not beql:HasQuestTimer(questID)) then
        return;
    end

    local questTime = beql:GetQuestTimer(questID) or 0;

    line = getQuestTrackerLine();
    line.type = 'quest';
    line.questID = questID;
    line:Show();
    line.statusBar:Show();
    line.statusBar.text:SetText(SecondsToTime(questTime));
    line.statusBar:SetMinMaxValues(0, questTime)
    line.statusBar:SetValue(questTime);
    line.mouseover:SetWidth(line.text:GetStringWidth())
end

function displayTrackerFrame()
    if (#beql.frames.watchLineCache.usedFrames==0) then
        -- nothing to display
        beql.frames.questWatchFrame:SetAlpha(0);
        beql.frames.questWatchFrame:SetHeight(0.001);
        return;
    end 

	-- Tracker height
	local height = 0 + beql.db.profile.QuestTracker.FontHeight; -- border spacing
	local width = beql.frames.questWatchTitle:GetWidth(); -- is calculated in header
	local linewidth = 0 
	local nextoffset = 0
	local previousLine;
    local spacing = beql.db.profile.QuestTracker.FontHeight;

	for k, line in ipairs(beql.frames.watchLineCache.usedFrames) do
        previousLine = k == 1 and beql.frames.questWatchFrame or beql.frames.watchLineCache.usedFrames[k - 1];
        nextoffset = (line.type == 'header' and 0.5 or line.type == 'quest' and 0.25 or 0.125) * line:GetHeight();
        linewidth = line.dash:GetWidth() + line.text:GetWidth() + spacing;

        if (not beql.db.profile.QuestTracker.AutoResize and linewidth > beql.db.profile.QuestTracker.FixedWidth) then
            line.text:SetWidth(beql.db.profile.QuestTracker.FixedWidth - line.dash:GetWidth() - spacing);
            linewidth = beql.db.profile.QuestTracker.FixedWidth;
        end

		if (k > 1) then
 			line:SetPoint("TOPLEFT", previousLine, "BOTTOMLEFT", 0, -nextoffset);
 			line:SetPoint("TOPRIGHT", previousLine, "BOTTOMRIGHT", 0, -nextoffset);
		else 
 			line:SetPoint("TOPLEFT", previousLine, "TOPLEFT", spacing / 2, (-spacing - 8) / 2); -- indent of 4
 			line:SetPoint("RIGHT", previousLine, "RIGHT", -(spacing / 2), (-spacing - 8) / 2); -- indent of 4
		end

        line.mouseover:Show();
        line.mouseover:ClearAllPoints();
        line.mouseover:SetPoint('LEFT', line.text);
        line.mouseover:SetPoint('RIGHT', line.text);
        line.mouseover:SetPoint('TOP', line, 'TOP', 0, nextoffset);
        line.mouseover:SetPoint('BOTTOM', line);

		height = height + line:GetHeight() + nextoffset;
        width = max(width, linewidth);
	end
	
	beql.frames.questWatchFrame:SetAlpha(beql.db.profile.QuestTracker.Alpha)
	beql.frames.questWatchFrame:SetHeight(height)
	beql.frames.questWatchFrame:SetWidth(width)
end

function getQuestTrackerLine()
    local line = beql.frames.watchLineCache:GetFrame();
	local t1, _ , t2 = line.text:GetFont()

	line:ClearAllPoints();
	line:SetHeight(beql.db.profile.QuestTracker.FontHeight * 1.125)
    line:Show();
	line.type = nil;

	line.dash:ClearAllPoints();
	line.dash:SetFont(t1, beql.db.profile.QuestTracker.FontHeight, t1);
    line.dash:SetPoint('TOPLEFT', line, 'TOPLEFT', 0, beql.db.profile.QuestTracker.FontHeight * -0.0625);
    line.dash:SetPoint('BOTTOMLEFT', line, 'BOTTOMLEFT', 0, beql.db.profile.QuestTracker.FontHeight * 0.0625);
	line.dash:SetText("");
	line.dash:SetTextColor(0.8, 0.8, 0.8);
	line.dash:Show();
	line.dash:SetWidth(0);

	line.statusBar:Hide();

	line.text:SetFont(t1, beql.db.profile.QuestTracker.FontHeight, t1);
	line.text:SetText("");
	line.text:SetTextColor(0.8, 0.8, 0.8);
	line.text:SetWidth(0)
	line.text:Show();

	return line;
end

function beql:QuestTracker_StatusBar_OnUpdate(elapsed)
	if (not self.statusBar:IsShown()) then
		-- stausbar not shown, exit
		return;
	end

	self.time = (self.time or 0) + elapsed
	if (self.time < 0.1) then
		-- only update every 0.1 seconds
		return;
	end
	
	if (not beql:HasQuestTimer(self.questID)) then
		self:Hide();
		-- ToDo: Update WatchFrame
		return;
	end
	
	local questtimeleft = beql:GetQuestTimer(self.questID);
	
	if (not questtimeleft or type(questtimeleft) ~= 'number') then
		self:Hide();
		
		return;
	end
	
	self.statusBar.text:SetText(SecondsToTime(questtimeleft));
	self.statusBar:SetValue(questtimeleft);
	self.time = 0
end

function beql:WatchFrameLine_OnUpdate(elapsed)
    --[[
    self.updateTime = (self.updateTime or 0.2) - elapsed;
	
	if (self.updateTime > 0) then
    return
	end
	
	self.updateTime = 0.2; 
	
	if (self:IsMouseOver()) then
    local line = self:GetParent()
    if (GetMouseFocus() == beql.frames.questWatchFrame) then
    beql.frames.questWatchFrame.questID = line.questID
    beql.frames.questWatchFrame.lastIdUpdate = GetTime() + .2
    end
	end
    --]]
end

function beql:watchFrame_OnDragStart()
    if (beql.frames.questWatchAnchor:IsMovable()) then
        beql.frames.questWatchAnchor.ismoving = true;
        beql.frames.questWatchAnchor:StartMoving();
    end
end

function beql:watchFrame_OnDragStop()
    beql.frames.questWatchAnchor.ismoving = false
    beql.frames.questWatchAnchor:StopMovingOrSizing()
    beql:SaveFramePosition(beql.frames.questWatchAnchor)
end

function beql:watchFrameLine_OnEnter(...)
    local line = self:GetParent();

    if (line.type ~= 'quest' and line.type ~= 'objective') then
        return;
    end


    if (beql.frames.questWatchTooltip.questID ~= line.questID) then
        beql.frames.questWatchTooltip.questID = line.questID;
    end

    beql:QuestTracker_OnUpdate(1);
end


function beql:watchFrameLine_OnLeave(...)
    local line = self:GetParent();
    
    if (beql.frames.questWatchTooltip.questID == line.questID) then
        beql.frames.questWatchTooltip.questID = nil;
    end 


    self.LeftButton = nil;
    self.RightButton = nil;
end

function beql:watchFrameLine_OnMouseDown(button)
    self[button] = GetTime();
end

function beql:watchFrameLine_OnMouseUp(button)
    local line = self:GetParent();
    local duration = GetTime() - (self[button] or GetTime());

    if (duration == 0) then
        return;
    end

    local _, _, _, _, _, _, _, questID, questIndex, headerID = beql:GetQuestLogTitle(line.questID);

    if (line.type == 'header' and button == 'LeftButton') then
        --@TODO: open map
        return;
    elseif (line.type == 'header' and button == 'RightButton') then
        local header = line.questID;
        for _, questID in pairs(beql:GetZoneQuests(line.questID)) do
            local childIndex = select(9, beql:GetQuestLogTitle(questID))
            if (IsQuestWatched(childIndex)) then
                beql:toggleQuestWatch(childIndex);
            end
        end
    elseif (IsShiftKeyDown() and button == 'LeftButton') then
        --@TODO: post quest
        return;
        elseif (button == 'LeftButton') then
            if (QuestLogFrame:IsVisible() ~= true) then
                securecall(ToggleQuestLog);
            end

            securecall(SelectQuestLogEntry, questIndex);

            if(questIndex > GetNumQuestLogEntries())then
                local headerIndex = select(9, beql:GetQuestLogTitle(headerID)) 
                securecall(ExpandQuestHeader, headerIndex);
            end
        elseif (button == 'RightButton') then
            beql:toggleQuestWatch(questIndex);
        end

    QuestLog_Update();
    beql:QuestTracker_DisplayQuests();
end

-- EOF --
